#line 1
precision highp float;

#include "shaders/library/common.shader"
#include "shaders/library/random.shader"
#include "shaders/library/brdf.shader"
#include "shaders/library/covariance.shader"
#line 8

// Uniform variables
uniform vec3  wi;
uniform float std_dev;

uniform float  u_Alpha2;
uniform float  u_Scale;
uniform float  u_Gamma;

/* Gamma transformation
 */
vec3 Gamma(vec3 R) {
    return exp(u_Gamma * log(u_Scale * R));
}

/* Evaluate an emissive Gaussian term
 */
vec3 Emission(vec3 w, float alpha) {
    return GGX(wi, w, vec3(0.0, 0.0, 1.0), alpha) * vec3(1,1,1);
}

void main(void) {

    // Get the omega_o direction from the vPos variable
    vec3 wo;
    wo.xy = vPos.xy;
    float sinTo = length(wo.xy);
    if(sinTo <= 1.0) {
        wo.z = sqrt(1.0 - sinTo*sinTo);

        /* Update the roughness in the linear space for GGX
         */
        float new_dev = varianceToRoughness(roughnessToVariance(std_dev) + roughnessToVariance(u_Alpha2));

        /* Evaluate the BRDF lobe
         */
        gl_FragColor.xyz = Gamma(Emission(wo, new_dev));
        gl_FragColor.w   = 1.0;

    } else {
        gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
    }
}