#line 1
precision highp float;

#include "shaders/library/common.shader"
#include "shaders/library/random.shader"
#include "shaders/library/brdf.shader"
#include "shaders/library/media.shader"
#include "shaders/library/covariance.shader"
#line 9

// Uniform variables
uniform vec3  wi;
uniform float std_dev;

uniform float  u_G;
uniform float  u_Scale;
uniform float  u_Gamma;
uniform float  u_Depth;
uniform float  u_Sigma_a;
uniform float  u_Sigma_s;
uniform int    u_VizuSelect;

/* Gamma transformation
 */
vec3 Gamma(vec3 R) {
    return exp(u_Gamma * log(u_Scale * R));
}

/* Evaluate an emissive term
 */
vec3 Emission(vec3 w, float dev) {
    return GGX(wi, w, vec3(0.0, 0.0, 1.0), dev) * vec3(1,1,1);
}

/* Return the inverse covariance coefficient to perfrom
 * 'convolution' in a linear space with the BRDF.
 */
float HG_variance(float g) {
    return pow((1.0-g)/g, 0.8) / (1.0+g); // (2) Best fit I could find
}

void main(void) {

    /* Constants */
    float c_Sigma_t = u_Sigma_a + u_Sigma_s;

    gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);

    // Get the omega_o direction from the vPos variable
    vec3 wo;
    wo.xy = vPos.xy;
    float sinTo = length(wo.xy);
    if(sinTo <= 1.0) {
        wo.z = sqrt(1.0 - sqr(sinTo));

        /* Evaluate the absorption */
        float Abs_wi = exp(- c_Sigma_t*u_Depth / wi.z);
        float Abs = Abs_wi;

        /* Change the roughness to match the scattering parameters */
        float old_var = roughnessToVariance(std_dev);
        float new_var = old_var + HG_variance(u_G);
        float new_dev = varianceToRoughness(new_var);

        /* Evaluate radiance */
        vec3 value = u_Sigma_s * u_Depth * Abs * Emission(wo, new_dev);

        /* Aggregate sample value */
        gl_FragColor.xyz += value;

        gl_FragColor.xyz = Gamma(gl_FragColor.xyz);
        gl_FragColor.w = 1.0;
    }
}
