#line 1
precision highp float;

#include "shaders/library/common.shader"
#include "shaders/library/random.shader"
#include "shaders/library/brdf.shader"
#include "shaders/library/layer.shader"
#line 8

// Uniform variables
uniform vec3  wi;

uniform float  std_dev;
uniform float  u_Alpha2;
uniform float  u_Scale;
uniform float  u_Gamma;

uniform float  u_Eta1;
uniform float  u_Eta2;

uniform float  u_KappaR;
uniform float  u_KappaG;
uniform float  u_KappaB;

// Pass number for the progressive renderer
uniform int u_PassNumber;

#define NB_SAMPLES 1024
#define NB_PER_RUN 1024

/* Gamma transformation
 */
vec3 Gamma(vec3 R) {
    return exp(u_Gamma * log(u_Scale * R));
}

/* Evaluate an emissive term
 */
vec3 Emission(vec3 w, float alpha) {
    return GGX(wi, w, vec3(0.0, 0.0, 1.0), alpha) * vec3(1,1,1);
}


void main(void) {

    gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);

    // Remove issues with the roughness
    float a = max(u_Alpha2, 0.001);

    // Get the omega_o direction from the vPos variable
    vec3 wo;
    wo.z = cos(0.25*PI*(1.0+vPos.x));
    wo.x = sqrt(1.0 - sqr(wo.z));
    wo.y = 0.0;

    for(int i=0; i<NB_PER_RUN; ++i) {

        /* Radom number
        */
        int  idx = mod(u_PassNumber*NB_PER_RUN+i, NB_SAMPLES);
        //int  idx = mod(i, NB_PER_RUN);
        //vec2 rnd = QMC_Davenport(idx, NB_SAMPLES);
        vec2 rnd = QMC_Additive_2D(idx);

        /* Generate a sample from the GGX distribution
        */
        vec3 m; float pdf;
        GGX_Sample_VNDF(wo, max(0.5*(vPos.y + 1.0), 0.01), rnd.xy, m, pdf);
        //m.z  = rnd.x;
        //m.xy = sqrt(1.0 - sqr(m.z)) * vec2(cos(2.0*PI*rnd.y), sin(2.0*PI*rnd.y));
        vec3 ws = -reflect(wo, m);

        /* Evaluate radiance */
        vec3 value = vec3(0,0,0);
        if(ws.z > 0.0) {
            /* Evaluate the Fresnel term */
            vec3 R;
            R.x = FresnelUnpolarized(dot(wo, m), u_Eta1, u_Eta2, u_KappaR);
            R.y = FresnelUnpolarized(dot(wo, m), u_Eta1, u_Eta2, u_KappaG);
            R.z = FresnelUnpolarized(dot(wo, m), u_Eta1, u_Eta2, u_KappaB);
            gl_FragColor.xyz += R;
        }
    }

    gl_FragColor.xyz = Gamma(gl_FragColor.xyz / float(NB_PER_RUN));
    gl_FragColor.w = 1.0;

}
