#line 1
precision highp float;

#include "shaders/library/common.shader"
#include "shaders/library/random.shader"
#include "shaders/library/brdf.shader"
#line 6

// Pass number for the progressive renderer
uniform int u_PassNumber;
uniform int u_RandomSequence;
uniform sampler2D u_Sampler;
uniform sampler2D u_Sampler2;
uniform float u_Time;
uniform vec3 wi;
#define NB_SAMPLES 256

const vec3 U = vec3(1,0,0);
const vec3 V = vec3(0,1,0);
const vec3 W = vec3(0,0,1);

vec3 uvToDirection(vec2 uv) {

    //vec3 w = uv.x*U + V - uv.y*W;

    float t = PI*(0.5*uv.y+0.5);
    float p = PI*uv.x;
    vec3 w = vec3(cos(p)*sin(t), sin(p)*sin(t), cos(t));
    return normalize(w);

}

vec4 Envmap(vec3 w) {

    float t = w.z;//sign(w.z)*acos(w.z);
    float p = atan(w.y, w.x);

    vec2 uv;
    uv.x = 0.5*p/PI + 0.5;
    uv.y = 0.5*t + 0.5;

    return texture2D(u_Sampler, uv);
}

bool intersectSphere(vec2 uv, out vec3 w, out vec3 p, out vec3 n) {
    if(length(uv) > 1.0) {
        return false;
    } else {
        p = vec3(uv.x, sqrt(1.0-sqr(length(uv))), uv.y);
        n = p;
        w = vec3(sin(2.0*PI*u_Time), cos(2.0*PI*u_Time), 0);
        return true;
    }
}

/* Test for displaying textures
 */
void main(void) {
    vec2 uv = vPos.xy;
    vec3 w, p, n;
    vec4 c = vec4(0,0,0,0);
    if(intersectSphere(uv, w, p, n)) {

        /* Construct the local coordinate frame */
        vec3 t, b;
        mat3 frame;
        FrameFrisvad(n, t, b);
        frame = mat3(t, b, n);

        /* Project the incoming vector */
        vec3 wl = w * frame;
        for(int idx=0; idx<NB_SAMPLES; ++idx) {
            vec3 h; float pdf;
            GGX_Sample_VNDF(wl, 0.04, QMC_Davenport(idx, NB_SAMPLES), h, pdf);

            /* Sample a new direction */
            vec3 rl = reflect(wl, h);
            w = frame * rl;

            // vec3 r = reflect(w, n);
            // w = r;
            c += Envmap(w) / float(NB_SAMPLES);
        }
    } else {
        w = uvToDirection(uv + wi.xy);
        c = Envmap(w);
    }
    gl_FragColor = c;
}