#line 1
precision highp float;

#include "shaders/library/common.shader"
#include "shaders/library/random.shader"
#include "shaders/library/brdf.shader"
#include "shaders/library/layer.shader"
#line 7

// Uniform variables
uniform vec3  wi;
uniform float std_dev;

uniform float  u_Scale;
uniform float  u_Gamma;
uniform float  u_Depth;
uniform int    u_VizuSelect;

/* Gamma transformation
 */
vec3 Gamma(vec3 R) {
    return exp(u_Gamma * log(u_Scale * R));
}

/* Evaluate an emissive Gaussian term
 */
vec3 Emission(vec3 w, float dev) {
    vec3 n = wi; n.xy = - n.xy;
    return TangentialGaussian(w, n, dev) * vec3(1,1,1);
}

void main(void) {

    // Get the omega_o direction from the vPos variable
    vec3 wo;
    wo.xy = vPos.xy;
    float sinTo = length(wo.xy);
    if(sinTo <= 1.0) {
        float cosTo = sqrt(1.0 - sinTo*sinTo);
        wo.z = cosTo;

        float absorption = exp(- u_Depth * sqrt(1.0 + sqr(sinTo/cosTo)));
        if(u_VizuSelect >= 1) {
            float sinTi = length(wi.xy);
            float cosTi = wi.z;
            absorption /= exp(- u_Depth);
        }

        /* Evaluate the input GGX lobe */
        float GGX_r;
        GGX_r  = GGX(wi, wo, vec3(0.0, 0.0, 1.0), max(std_dev, 1.0E-5));

         /* Perform random evaluation of the layered structure using 'wiT' and 'woT'
          * as the incoming and outgoing directions.
          */
        if(wo.z == 0.0) {
            gl_FragColor.xyz = vec3(0.0, 0.0, 0.0);
        } else {
            gl_FragColor.xyz = Gamma(absorption * GGX_r * vec3(1,1,1));
            if(u_VizuSelect >= 2) {
                gl_FragColor.xyz = mod(10.*gl_FragColor.xyz, 1.0);
            }
        }
        gl_FragColor.w   = 1.0;

    } else {
        gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
    }
}