#line 1
precision highp float;

#include "shaders/library/common.shader"
#include "shaders/library/random.shader"
#include "shaders/library/brdf.shader"
#line 7

// Uniform variables
uniform vec3  wi;

uniform float  std_dev;
uniform float  u_Alpha2;
uniform float  u_Scale;
uniform float  u_Gamma;

// Pass number for the progressive renderer
uniform int u_PassNumber;

#define NB_SAMPLES 25600
#define NB_PER_RUN 256

/* Gamma transformation
 */
vec3 Gamma(vec3 R) {
    return exp(u_Gamma * log(u_Scale * R));
}

/* Evaluate an emissive term
 */
vec3 Emission(vec3 w, float alpha) {
    return GGX(wi, w, vec3(0.0, 0.0, 1.0), alpha) * vec3(1,1,1);
}

void main(void) {

    gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);

    // Get the omega_o direction from the vPos variable
    vec3 wo;
    wo.xy = vPos.xy;
    float sinTo = length(wo.xy);
    if(sinTo <= 1.0) {
        wo.z = sqrt(1.0 - sqr(sinTo));

        for(int i=0; i<NB_PER_RUN; ++i) {

            /* Radom number
            */ 
            int  idx = mod(u_PassNumber*NB_PER_RUN+i, NB_SAMPLES);        
            vec2 rnd = QMC_Additive_2D(idx);

            /* Generate a sample from the GGX distribution
            */
            vec3 m; float pdf;
            GGX_Sample_VNDF(wo, u_Alpha2, rnd.xy, m, pdf);
            vec3 ws = reflect(wo, m); ws.z = -ws.z;

            /* Evaluate radiance
             */
            vec3 value = vec3(0,0,0);
            if(ws.z > 0.0) {
                value = Emission(ws, std_dev);
            }

            /* Perform random evaluation of the layered structure using 'wiT' and 'woT'
            * as the incoming and outgoing directions.
            */
            gl_FragColor.xyz += value;
        }

        gl_FragColor.xyz = Gamma(gl_FragColor.xyz / float(NB_PER_RUN));
        gl_FragColor.w = 1.0;
    }
}