#line 1
precision highp float;

#include "shaders/library/common.shader"
#include "shaders/library/random.shader"
#include "shaders/library/brdf.shader"
#include "shaders/library/media.shader"
#include "shaders/library/layer.shader"
#line 7

// Uniform variables
uniform vec3  wi;
uniform float std_dev;

uniform float  u_G;
uniform float  u_Scale;
uniform float  u_Gamma;
uniform float  u_Depth;
uniform float  u_Sigma_a;
uniform float  u_Sigma_s;
uniform int    u_VizuSelect;

// Pass number for the progressive renderer
uniform int u_PassNumber;

#define NB_SAMPLES 1024
#define NB_PER_RUN 1

/* Gamma transformation
 */
vec3 Gamma(vec3 R) {
    return exp(u_Gamma * log(u_Scale * R));
}

/* Evaluate an emissive term
 */
vec3 Emission(vec3 w, float dev) {
    return GGX(wi, w, vec3(0.0, 0.0, 1.0), max(dev, 1.0E-5)) * vec3(1,1,1);
}

void main(void) {

    /* Constants */
    float c_Sigma_t = u_Sigma_a + u_Sigma_s;

    gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);

    // Get the omega_o direction from the vPos variable
    vec3 wo;
    wo.xy = vPos.xy;
    float sinTo = length(wo.xy);
    if(sinTo <= 1.0) {
        wo.z = sqrt(1.0 - sqr(sinTo));

        for(int i=0; i<NB_PER_RUN; ++i) {

            /* Radom number
             */
            int  idx = mod(u_PassNumber*NB_PER_RUN+i, NB_SAMPLES);
            vec3 rnd = QMC_Additive_3D(idx);

            /* Generate a Frame a sample the HG distribution
             */
            vec3 t, b, ws;
            FrameFrisvad(wo, t, b);
            HG_Sample(u_G, rnd.xy, ws);
            ws = ws.x*t + ws.y*b + ws.z*wo;

            /* Sample a distance
             */
            float dist  = -log(1.0 - rnd.z) / c_Sigma_t;
            float depth = dist * wi.z;

            vec3 value = vec3(0,0,0);
            if(ws.z > 0.0 && depth < u_Depth) {
                /* Evaluate transmittance */
                float Abs_ws = exp(- (u_Depth-depth) * c_Sigma_t / ws.z);
                float Abs = Abs_ws;

                /* Evaluate radiance */
                value = u_Sigma_s * Abs * Emission(ws, std_dev);
            }

            /* Aggregate random sample */
            gl_FragColor.xyz += value;
        }

        gl_FragColor.xyz = Gamma(gl_FragColor.xyz / float(NB_PER_RUN));
        gl_FragColor.w = 1.0;
    }
}
