#line 1
precision highp float;

#include "shaders/library/common.shader"
#include "shaders/library/random.shader"
#include "shaders/library/brdf.shader"
#include "shaders/library/layer.shader"
#line 7

// Uniform variables
uniform vec3  wi;
uniform float roughness;
uniform float eta1;

// Pass number for the progressive renderer
uniform int u_PassNumber;
uniform float  u_Scale;
uniform float  u_Gamma;

/* Gamma transformation
 */
vec3 Gamma(vec3 R) {
    return exp(u_Gamma * log(u_Scale * R));
}

/* Evaluate the transmitted light-field of the first interface
 */
vec3 Transmitted(in vec3 wC, in vec3 wL) {

    vec3 wT = -wL;
    wT.z = wT.z;

    // Get layers
    BsdfLayer layer1 = GetLayer(0);
    BsdfLayer layer2 = GetLayer(1);

    return GGX_T(wC, wT, vec3(0.0, 0.0, 1.0), layer1.n, layer2.n, max(layer2.a, 1.0E-5)) * vec3(1.0, 1.0, 1.0);
}

void main(void) {

    // Get the omega_o direction from the vPos variable
    vec3 wo;
    wo.xy = -vPos.xy;
    float sinTo = length(wo.xy);
    if(sinTo <= 1.0) {
        wo.z = sqrt(1.0 - sinTo*sinTo);

        vec3 wr = wi;
        wr.xy = - wr.xy;

        // Evaluate the transmitted ray
        vec3 wt = SnellRefraction(wi, vec3(0.0, 0.0, 1.0), layers[0].n/layers[1].n);

        // Update the mean direction to match the off-specular direction
        float smoothness = 1.0 - layers[1].a;
        float lerpFactor = smoothness * (sqrt(smoothness) + layers[1].a);
        vec3  wa = lerpFactor*wt + (1.0-lerpFactor)*wi;
        wa.z = sqrt(1.0 - (wa.x*wa.x + wa.y+wa.y));

         /* Perform random evaluation of the layered structure using 'wiT' and 'woT'
          * as the incoming and outgoing directions.
          */
        gl_FragColor.xyz = Gamma(Transmitted(wi, wo));
        gl_FragColor.w   = 1.0;

        // Display helpers that show the incoming ray and the purely reflected
        // ray using circles.
        float dist;
        dist = length(wo.xy - wt.xy);
        if(dist < 0.01) {
            gl_FragColor.xyz = vec3(0.0, 0.0, 1.0);
        }
        dist = length(wo.xy - wr.xy);
        if(dist < 0.01) {
            gl_FragColor.xyz = vec3(0.0, 1.0, 0.0);
        }
        dist = length(wo.xy - wi.xy);
        if(dist < 0.01) {
            gl_FragColor.xyz = vec3(1.0, 0.0, 0.0);
        }
        dist = length(wo.xy - wa.xy);
        if(dist < 0.01) {
            gl_FragColor.xyz = vec3(1.0, 0.0, 1.0);
        }
        
    } else {
        gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
    }
}