#line 1
precision highp float;

#include "shaders/library/common.shader"
#include "shaders/library/random.shader"
#include "shaders/library/brdf.shader"
#include "shaders/library/layer.shader"
#line 7

// Uniform variables
uniform vec3  wi;
uniform float std_dev;

uniform float  u_Scale;
uniform float  u_Gamma;

/* Gamma transformation
 */
vec3 Gamma(vec3 R) {
    return exp(u_Gamma * log(u_Scale * R));
}
/* Evaluate an emissive GGX term
 */
vec3 Emission(vec3 _wi, vec3 _wo, float _dev) {
    return GGX(_wi, _wo, vec3(0.0, 0.0, 1.0), _dev) * vec3(1,1,1);
}

void main(void) {

    // Get the omega_o direction from the vPos variable
    vec3 wo;
    wo.xy = vPos.xy;
    float sinTo = length(wo.xy);
    if(sinTo <= 1.0) {
        wo.z = sqrt(1.0 - sinTo*sinTo);

        /* Evaluate the ratio of IORs
         */
        float ior   = layers[0].n/layers[1].n;
        float scale = 1.0/ior;

        /* Perform random evaluation of the layered structure using 'wiT' and 'woT'
         * as the incoming and outgoing directions.
         */
        gl_FragColor.xyz = Gamma(Emission(wi, wo, std_dev));
        gl_FragColor.w   = 1.0;

    } else {
        gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
    }
}